/*
 *  xloader/include/onenand.h
 *
 *  OneNAND header file
 *
 *  Copyright(C) 2005 Samsung Electronics
 *  Kyungmin Park <kyungmin.park@samsung.com>
 */

#ifndef __ONENAND_H
#define __ONENAND_H

#include "onenand_regs.h"

#define onenand_readw(a)	(*(volatile unsigned short *)(a))
#define onenand_writew(v, a)	((*(volatile unsigned short *)(a)) = (u16) (v))

#define THIS_ONENAND(a)		(ONENAND_ADDR + (a))

#define READ_INTERRUPT()						\
	onenand_readw(THIS_ONENAND(ONENAND_REG_INTERRUPT))

/*
 * Architecture specific memory configurations for Sync. Burst Read
 */
#if defined(CFG_SYNC_BURST_READ)
#if defined(CONFIG_OMAP1610)

#define SET_EMIFS_CS_CONFIG(v)						\
	(*(volatile unsigned long *)(OMAP_EMIFS_CS_CONFIG) = (v))

static inline void onenand_sync_read(void)
{
	unsigned int value;
	value = 0
		| ONENAND_SYS_CFG1_SYNC_READ	/* Read Mode: Synchronous */
		| ONENAND_SYS_CFG1_BRL_4	/* Burst Read Latency: 4 */
		| ONENAND_SYS_CFG1_BL_8		/* Burst Length : 8 words*/
		| ONENAND_SYS_CFG1_RDY		/* RDY signal plarity */
		| ONENAND_SYS_CFG1_INT		/* INT signal plarity */
		| ONENAND_SYS_CFG1_IOBE		/* I/O buffer enable */
		;
	onenand_writew(value, THIS_ONENAND(ONENAND_REG_SYS_CFG1));

	value = 0
		| (4 << 16)		/* Synchronous Burst Read */
		| (1 << 12)		/* PGWST/WELEN */
		| (1 << 8)		/* WRWST */
		| (4 << 4)		/* RDWST */
		| (1 << 0)		/* FCLKDIV => 48MHz */
		;
	SET_EMIFS_CS_CONFIG(value);
}

static inline void onenand_async_read(void)
{
	unsigned int value;
	value = 0
		| (0x0 << 15)			/* Read Mode: Asynchronous */
		| ONENAND_SYS_CFG1_BRL_4	/* Burst Read Latency : 4 */
		| ONENAND_SYS_CFG1_BL_CONT	/* Burst Length : continuous */
		| ONENAND_SYS_CFG1_RDY		/* RDY signal plarity */
		| ONENAND_SYS_CFG1_INT		/* INT signal plarity */
		| (0x0 << 5)			/* I/O buffer disable */
		;

	onenand_writew(value, THIS_ONENAND(ONENAND_REG_SYS_CFG1));

	value = 0
		| (0 << 16)		/* Asynchronous Read */
		| (1 << 12)		/* PGWST/WELEN */
		| (1 << 8)		/* WRWST */
		| (3 << 4)		/* RDWST */
		| (1 << 0)		/* FCLKDIV => 48MHz */
		;
	SET_EMIFS_CS_CONFIG(value);
}

#elif defined(CONFIG_PXA27X)

#include <asm/arch/pxa-regs.h>

#define SET_SXCNFG(v)		((SXCNFG) = (v))

#define SXCNFG_DEFAULT			(0x40044004)
#define SXTP0_SYNC_BURST_8		(2 << 12)
#define SXTP0_SYNC_BURST_16		(3 << 12)
#define SXCL0_4				(3 << 2)
#define SXCL0_5				(4 << 2)
#define SXEN0				(1 << 0)

static inline void onenand_sync_read(void)
{
	unsigned int value;
	value = 0
		| ONENAND_SYS_CFG1_SYNC_READ	/* Read Mode: Synchronous */
		| ONENAND_SYS_CFG1_BRL_4	/* Burst Read Latency: 4 */
		| ONENAND_SYS_CFG1_BL_16	/* Burst Length : 16 words*/
		| ONENAND_SYS_CFG1_RDY		/* RDY signal plarity */
		| ONENAND_SYS_CFG1_INT		/* INT signal plarity */
		| ONENAND_SYS_CFG1_IOBE		/* I/O buffer enable */
		;
	onenand_writew(value, THIS_ONENAND(ONENAND_REG_SYS_CFG1));

	value = 0
		| SXTP0_SYNC_BURST_16
		| SXCL0_4
		| SXEN0
		;
	SET_SXCNFG(value);
}

static inline void onenand_async_read(void)
{
	unsigned int value;
	value = 0
		| (0x0 << 15)			/* Read Mode: Asynchronous */
		| ONENAND_SYS_CFG1_BRL_4	/* Burst Read Latency : 4 */
		| ONENAND_SYS_CFG1_BL_CONT	/* Burst Length : continuous */
		| ONENAND_SYS_CFG1_RDY		/* RDY signal plarity */
		| ONENAND_SYS_CFG1_INT		/* INT signal plarity */
		| (0x0 << 5)			/* I/O buffer disable */
		;

	onenand_writew(value, THIS_ONENAND(ONENAND_REG_SYS_CFG1));

	/* Reset value */
	value = SXCNFG_DEFAULT;
	SET_SXCNFG(value);
}

#else
#error Your boards are not supported.
#endif

#else	/* !CFG_SYNC_BURST_READ */
#define onenand_sync_read()		do { } while (0)
#define onenand_async_read()		do { } while (0)
#endif	/* CFG_SYNC_BURST_READ */

extern int onenand_read_block(unsigned char *buf, ulong block);

#endif	/* __ONENAND_H */
